(* ::Package:: *)

(* moeb2.m *)

(* Developing Version *)

(* Created by Rolf Sulanke *)

(* Started December 28, 2010, finished July 26, 2012*)

(* This package contains tools applied in notebooks about 
M\[ODoubleDot]bius geometry, mainly in three or two dimensions. These and other functions
are systematically developed and applied in my notebook emg.nb,
 "Elementary Moebius Geometry I, Points and Spheres."
ATTENTION! Changes of function else defined in the package moeb.m; 
confeuklid, euklid.*)

(* Hint:s. 

1. Be sure, that besides of this package also the packages neuvec.m, euvec.m, mspher.m, and 
vectorcalc.m are contained in the working path $Path.

2. After importing this package define the numbers
dim (= dimension) and ind (=index) of the pseudo-euclidean vector space. They  are 
defined as constants within the Global context of the notebook!!! These 
numbers may be changed later in the notebook, if wanted. If they are defined before 
importing the notewbook they will be fixed for all the modules introduce in the packages!*)

(* Free software: Feel free to apply, distribute or change my packages and
notebooks. Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)



BeginPackage["moeb2`",{"vectorcalc`", "euvec`", "neuvec`","mspher`", "mcirc`", "eudiffgeo`"}] 
   
(* The usages *)
(* Pseudo-Euclidean Geometry *)

iob::usage = 
  "iob[i,d:dim], 0<=i<=d -1, is the isotropic-orthonormal standard \
basis of the pseudo-Euclidean vector space of dimension dim and index \
1, numbered from 0 to dim -1, expressed in pseudo-orthogonal coordinates. ";

io::usage = 
  "io[] is the Gram matrix of the scalar products of index 1 and dimension dim>1 in the \
dim-dimensional pseudo-Euclidean vector space of the isotropic orthonormal 
standard basis.";

iodual::usage = "iodual[v_] is the dual vecor of v with respect to the 
isotropic-orthogonal coordinates in the pseudo-orthogonal vector space of index 1 
and dim >1.";

iosp::usage =
"iosp[v,w] is the scalar product of the vectors v,w of the 
d-dimensional pseudo-Euclidean space of index 1, d>1; 
v, w given in isotropic-orthonormal coodinates.";

ioCross::usage = "ioCross[v(1),...,v(d-1)] is the vector product  of 
d -1 vectors in the d-dimensional pseudo-orthogonal vector space of index 1 
and dimension d>1 (in isotropic-orthogonal coordinates).";

iocross::usage = "iocross[vl] is the vector product of a list of 
dimenion -1 vectors in the pseudo-orthogonal vector space of index 1
and dimension d>1 (in isotropic-orthogonal coordinates).";
  
transio::usage = 
  "transio[] is the matrix transforming the pseudo-orthonormal \
coordinates of a pseudo-Euclidean vector space of dimension dim>1 and \
index 1 to the corresponding isotropic orthonormal coordinates.";

transoi::usage = 
  "transoi[] is the matrix transforming the  isotropic-orthonormal \
coordinates of a pseudo-Euclidean vector space of dimension dim >1 and \
index 1 to the corresponding pseudo-orthonormal basis.";

iomatrix::usage = "iomatrix[c] = iomatrix[vec[c,10]is the matrix of the general element of the Lie algebra o(1,4) in 
isotropic-orthogonal coordinates with the parameters c[1],...,c[10].";

iobas::usage = "iobas[i], i= 1,...,10, is the standard basis of the Lie algebra o(1,4) 
in isotropic-orthonormal coordinates.";

trl::usage = "trl[i,d;dim] is the i-th basis element of the Lie subalgebra of the 
Lie algebra of the Moebius group in dimension d-2, corresponding to the subgroup 
of translations; the matrices are expressed in isotropic-orthogonal coordinates.";

psomatrix::usage = "psomatrix[c]=psomatrix[vec[x,10]] is the matrix of the general element of the Lie algebra o(1,4) 
in pseudo-orthonormal coordinates with the parameters c[1],...,c[10].";

psobas::usage = "psobas[i], i= 1,...,10, is the standard basis of the Lie algebra o(1,4) in pseudo-orthonormal coordinates."

(* Euclidean Geometry *)

confeuklid::usage = "confeuklid[ulist] yields the standard isotropic vector of the (n+2)-dimensional 
pseudo-Euclidean space in isotropic-orthogonal coordinates representing the point with orthonormal 
coordinates u of the n-dimensional Euclidean space.";

euklid::usage = "euklid[ulist] is the point of the Euclidean space corresponding to the isotropic vector u; 
u must be given in isotropic-orthogonal coordinates; it is the inverse of confeuklid. 
WARNING! The result is correct for isotropic ulist only!";

euclidpt::usage = "euclidpt[u] yields the point of the Euclidean n-space 
which correspond to the point of the M\[ODoubleDot]bius n-space represented by the 
isotropic (n+2)-vector u under stereographic projection from the North pole. 
u must be given in isotropic-orthogonal coordinates.";

sphrefl::usage = "sphrefl[m]=sphericalreflection[m,o,2^(1/4)]is this special 
spherical reflection of  the point m at the origin o. It effects the transition 
between the functions invstepro and confeuklid. Since the origin is mapped 
to Infinity it strongly, but conformally distorts the neighbourhood of the origin.";

iohspherevec::usage = "iohspherevec[mList,r] is the spacelike unit vector in 
isotropic orthonormal coordinates corresponding to the hypersphere with radius r 
and center mList in the Euclidean space.";

ioradius::usage = "ioradius[v] is the radius of the hypersphere (circle in case of the 
Moebius plane) corresponding to the spacelike vector v in isotropic orthonormal 
coordinates.";

iocenter::usage = "iocenter[v] is the center of the hypersphere (circle in case of 
the Moebius plane) corresponding to the spacelike vector v in isotropic orthonormal 
coordinates.";

(* Spherical Geometry *)

conemap::usage = "conemap[ulist] maps the vector u of the n-dimenssional Euclidean vector space into the isotropic 
cone of its orthogonal (n+1)-dimensional  pseudo-Euclidean extension; restricted to the standard unit hypersphere 
it is a bijection between the points of the hypersphere and their normed representatives.";

sphmap::usage = "sphmap[ulist]maps the isotropic vector u onto the 
corresponding point of the unit hypersphere. Warning: The index 
must equal 1, and the result is valid for isotropic vectors in 
pseudo-orthogonal coordinates only!";

ccell::usage = "ccell[a,b] is the normal form of an infinitesimal elliptic transformation 
depending on the parameters a,b.";

grell::usage =  "grell[a,b,u,v] is a parameter representation of a 2-dimensional abelian subgroup 
of O[4] depending on constants a,b, with parameters u,v. grell[a,b,t]= grell[a,b,t,t]
is the 4x4-matrix of a one-parametric subgroup of the orthogonal group O[4], 
the isometry group of the unit 3-sphere, depending on the parameters a,b.";

ptell::usage = "ptell[u,v,w] is a parameter representation of the unit 3-sphere 
in the Euclidean 4-space.";

confell::usage = "confell[u_List] maps the n-dimensional hypersphere u.u= r^2  
of the (n+1)-dimensional Euclidean vector subspace pssp[u,stb[1]]==0 onto 
the isotropic representation of the  Moebius space (in pseudo-orthonormal coordinates) 
(n = dim -2).";


(* Hyperbolic Geometry *)

confhyp::usage = "For points of the pseudo-Euclidean model of the hyperbolic n-space in 
the (n+1)-dimensional pseudo-Euclidean space with pseudo-orthogonal coordinates confhyp[x] 
yields its standard isotropic representative as a point of the n-dimensional Moebius space.";

pthyp3D::usage ="pthyp3D[u,v,w,r:1] is a parameter representation  of the negative sheet of the hyper-hyperboloid 
pssp[x,x] = -1/r^2 in the Lorentz space (in pseudo-orthonormal coordinates); u,v,w are polar coordinates.";

hyp3D::usage = "hyp3D[u,v,w,r:1],0 <= u <= 2*Pi, 0<=v<=Pi, 0<=w, is the 5-dimensional isotropic representative
 of the hyperbolic point pthyp3D[u,v,w,r:1].";

hypunit::usage = "hypunit[n][{s1,...,sn}] is a parameter representation of the n-dimensional hyperbolic 
space of curvature -1 as the lower sheet of a two-sheeted hyper-hyperboloid in the (n+1)-dimensional 
pseudo-Euclidean vector space of index 1, sometimes called the unit hypersphere of imaginary radius i.";

hline::usage = "hline[ulist,vlist][t] is a parameter representation with parameter t of the 
hyperbolic line joining the 
hyperbolic points u, v in the pseudo-Euclidean model of the hyperbolic n-space.";

hplane3pts::usage = " If a,b,c are three linearly independent vectors of the (n+1)-dimensional 
pseudo-Euclidean vector space of index 1 spanning a pseudo-Euclidean subspace, then 
hplane3pts[a,b,c][u,v] is a parameter representation 
of the hyperbolic plane in the isometric pseudo-Euclidean model of the hyperbolic n-space.";



hypklein::usage = "hypklein[u,v,w]is the point with polar coordinates u,v,w 
in the unit ball, considered as the Klein conformal model of the 
hyperbolic 3-space; 0<=u<= 2Pi, 0<=v<=Pi, 0<= w."

sphhyp3D::usage = "sphhyp3D[u,v,w] is the point of the upper half of the unit 3-sphere 
corresponding to the point of the imaginary unit hypersphere in the Minkowski space 
with polar coordinates u,v,w.";

pthyp2D::usage ="pthyp2D[u,v,r:1] is a parameter representation of the positive sheet 
of a hyperboloid (in pseudo-orthonormal coordinates); u,v are polar coordinates.";

hyp2D::usage ="hyp2D[u,v,r] is a parameter representation of representatives 
of the points of the hyperbolic plane of curvature -1/r^2 in the M\[ODoubleDot]bius sphere 
(in pseudo-orthonormal coordinates); u,v are polar coordinates.";

sphhyp2D::usage = "sphhyp2D[u,v] is the point of the lower half of the unit sphere 
corresponding to the point of the imaginary unit sphere with polar coordinates u,v 
in the 3-dimensional Minkowski space.";

cchyp::usage = "cchyp[a,b] is the normal form of an infinitesimal Lorentz transformation
in the 4-dimensional Minkowski space depending on the parameters a,b.";

grhyp::usage = "grhyp[a,b,t] is the 4x4-matrix of a one-parametric subgroup of 
the pseudo-orthogonal group O[1,3], the Lorentz group of special relativity, 
depending on the parameters a,b.";



(* Moebius Geometry *)

mfre3D::usage = "mfre3D[k,h] is the matrix of the Moebius Frenet formulae 
with curvatures k, h in the three-dimensional Moebius space.";

mgcurve::usage = "mgcurve[cu][t] is an isotropic representative of the Euclidean curve 
cu[t]\[Element] \!\(\*SuperscriptBox[\(E\), \(n\)]\) in the (n+2)-dimensional pseudo-Euclidean vector space of n-dimensional 
Moebius geometry in isotropic-orthonormal coordinates.";


Begin["`Private`"]

(* Pseudo-Euclidean Geometry *)

dimm = Global`dim;
 
indd = Global`ind; 

iob[i_,d_:dimm] := Module[{},
  If[i >= 1 && i < d - 1, Return[stb[i + 1,d]]];
  If[i == 0, Return[(stb[1,d] - stb[d,d])/Sqrt[2]]];
  If[i == d - 1, Return[(stb[1,d] + stb[d,d])/Sqrt[2]]];
  Print["Incorrect argument!"]; Return[]];

io[d_:dimm] := If[dimm>1, Return[Append[Prepend[Table[stb[i,d],
{i,2,d-1}],-stb[d,d]],-stb[1,d]]], Print["Invalid dimension: ", dim < 2];
Return[]];

iodual[xx_]:= io[].xx

iosp[x_List,y_List]:= Module[{nn=Length[x]},
If[Not[Length[y]==nn]||(nn<2), Print["Invalid arguments!"];Return[]];
x.io[nn].y]
(*iosp[v_List,w_List, d_:dimm] := v.io[d].w; Changed Sept 9,2019. *)

iocross :=Composition[iodual,cross];

ioCross :=Composition[iodual,Cross];

transio[d_:dimm] := Table[iob[i,d], {i, 0, d - 1}];

transoi[d_:dimm ] := Transpose[transio[d]];

iomatrix[c_]:=iomatrix[vec[c,10]];

iomatrix[c_List] := Module[{n},
n = Length[c];
If[n< 10, Print["Wrong argument!"]; Return[]];
{{-c[[10]],c[[1]], c[[2]], c[[3]], 0},  {c[[7]], 0, c[[4]], c[[5]], c[[1]]}, {c[[8]], -c[[4]], 0, 
    c[[6]], c[[2]]}, {c[[9]], -c[[5]], -c[[6]], 0, c[[3]]}, 
   {0, c[[7]], c[[8]], c[[9]], c[[10]]}}];

iobas[i_] := iomatrix[stb[i,10]];

psomatrix[a_]:= psomatrix[vec[a,10]];
psomatrix[a_List] := Module[{n},
n = Length[a];
If[n< 10, Print["Wrong argument!"]; Return[]];
{{0,a[[1]],a[[2]],a[[3]],a[[10]]},{a[[1]],0,-a[[4]],-a[[5]],-a[[6]]},
{a[[2]],a[[4]],0,-a[[7]],-a[[8]]},{a[[3]],a[[5]],a[[7]],0,-a[[9]]},
{a[[10]],a[[6]],a[[8]],a[[9]],0}}];

psobas[i_] := psomatrix[stb[i,10]];
trl[i_,d_:dim]:= Module[{b = nullmatrix[d]},
b[[i+1]] = stb[1,d];b[[d]] = stb[i+1,d]; Return[b]]

(*Euclidean Geometry *)
(* The original confeuklid in moeb.m:
confeuklid[v_] := Module[{n},
If[Not[Head[v]== List],Print["False argument: ",v];Return[]];
n=Length[v];
If[v.v===\[Infinity],Return[stb[n+2,n+2]]];
Append[Prepend[v,1],v.v/2]];
replaced by *)

confeuklid[v_] := Module[{n},
If[Not[Head[v]== List],Print["False argument: ",v];Return[]];
n=Length[v];
If[v.v===\[Infinity],Return[stb[n+2,n+2]]];
Prepend[Append[v,1],v.v/2]];

(* The original euklid in moeb.m:
euklid[v_] := Module[{n},
If[!Head[v]===List,Print["False argument: ",v];Return[]];
n=Length[v];
If[N[v[[1]]]==0.`,Return[Table[\[Infinity],{n-2}]]];
Return[Take[Take[v/v[[1]],4],-3]]];
replaced by *)

euklid[v_] := Module[{n},
n=Length[v];
If[(!Head[v]===List)||n<3,Print["False argument: ",v];Return[]];
If[N[v[[n]]]==0.,Return[Table[\[Infinity],{n-2}]]];
Return[Take[Take[v/v[[n]],-(n-1)],n-2]]];

euclidpt[u_List] := sterproj[sphmap[transoi[].u]];

sphrefl[m_] := sphericalreflection[m,null[dimm-2],2^(1/4)];

iohspherevec[m_List,r_]:=-Append[Prepend[m,(-r^2+m.m)/2],1]/r;

ioradius[v_] := Module[{dd=Length[v],w},
If[v[[dd]]== 0||iosp[v,v]<=0||dd<2,Print["False argument!"];Return[]];
w =v/Sqrt[iosp[v,v]];
1/w[[dd]]];

iocenter[v_] := Module[{dd=Length[v],w},
If[v[[dd]]== 0||iosp[v,v]<=0||dd<2,Print["False argument!"];Return[]];
w =v/Sqrt[iosp[v,v]];
Take[w,{2,dd-1}]/w[[dd]]];


(* Spherical Geometry *)

conemap[u_List] := Prepend[u,Simplify[Sqrt[u.u]]];

sphmap[u_List ]:=Take[u/u[[1]],-Length[u]+1];

ccell[a_,b_] :={{0,a,0,0},{-a,0,0,0},{0,0,0,b},{0,0,-b,0}};

grell[a_,b_,u_,v_] := {{Cos[a*u],Sin[a*u],0,0},{-Sin[a*u],Cos[a*u],0,0},
{0,0,Cos[b*v],Sin[b*v]},{0,0,-Sin[b*v],Cos[b*v]}};

grell[a_,b_,t_] := grell[a,b,t,t];

ptell[u_,v_,w_]:= {Sin[w]*Cos[u]*Cos[v],Sin[w]*Cos[u]*Sin[v],Sin[w]*Sin[u],Cos[w]};

confell[v_List] := Prepend[v,Sqrt[v.v]];

(* Hyperbolic Geometry *)

confhyp[u_List] := Module[{nn = Length[u], rr,i},
If[nn<2,Print["Argument false!"];Return[]];
rr =-u[[1]]^2 + Sum[u[[i]]^2,{i,2,nn}];
If[rr>= 0, Print[u, "is not timelike!"]; Return[]];
Simplify[Append[u, Sqrt[-rr]]]];

pthyp3D[u_, v_, w_, r_: 1] := {-Cosh[w], Cos[u]*Cos[v]*Sinh[w], 
    Cos[v]*Sin[u]*Sinh[w], Sin[v]*Sinh[w]}*r;

hyp3D[u_, v_, w_, r_:1] := {-r*Cosh[w], r*Cos[u]*Cos[v]*Sinh[w], 
   r*Cos[v]*Sin[u]*Sinh[w],r*Sin[v]*Sinh[w], Sqrt[r^2]};

hypunit[n_][s_List] := Prepend[unit[n-1][Take[s,n-1]]*Sinh[s[[n]]],-Cosh[s[[n]]]];

(*confhyp3D[u_,v_,w_] := Append[hyp3D[u,v,w],1];*)

hline[u_List,v_List][t_] := Module[{nn = Length[u],w,a},
If[Length[v]!= nn,Print["Incorrect arguments!"];Return[]];
If[Chop[pssp[u,u]+1]!= 0.||Chop[pssp[v,v]+1]!= 0.,Print["Incorrect arguments!"];Return[]];
If[MatrixRank[{u,v}]<2,Print["Points are equal!"]; Return[]];
a =Simplify[pssp[u,v]];
w=normalize[v+u*a];
u*Cosh[t] + w*Sinh[t]];

hplane3pts[a_List,b_List,c_List][u_,v_]:= Module[{nn=Length[a],ww,www,uu,vv},
If[Length[b]!= nn||Length[c]!= nn, Print["Incorrect arguments!"];Return[]];
If[rank[{a,b,c}]<3, Print["Points are not in general position!"];Return[]];
ww = chsort[orthonorm[{a,b,c}]];
If[pssp[ww[[1]],ww[[1]]]>= 0,Print["Plane is not hyperbolic!"];Return[]];
www[uu_,vv_]=ww[[1]]*Cosh[vv]+ww[[2]]*Cos[uu]*Sinh[vv]+ww[[3]]*Sin[uu]*Sinh[vv];
If[www[0,0][[1]]>0,Return[-www[u,v]],Return[www[u,v]]]];

hypklein[u_,v_,w_] :=  {Cos[u]*Cos[v]*Tanh[w/2], 
    Cos[u]*Sin[v]*Tanh[w/2], 
    Sin[u]*Tanh[w/2]};

sphhyp3D[u_,v_,w_] :=-sphmap[hyp3D[u,v,w]]

pthyp2D[u_,v_,r_:1]:= {r Cosh[v],r Cos[u] Sinh[v],r Sin[u] Sinh[v]}

hyp2D[u_,v_,r_:1] := r*{Cosh[v], Sinh[v]Cos[u],Sinh[v]*Sin[u],1}

sphhyp2D[t_,u_] := sphmap[hyp2D[t,u,1]]

cchyp[a_,b_] :={{0,a,0,0},{a,0,0,0},{0,0,0,b},{0,0,-b,0}}

grhyp[a_,b_,t_] := {{Cosh[a t],Sinh[a t],0,0},{Sinh[a t],Cosh[a t],0,0},{0,0,Cos[b t],Sin[b t]},
{0,0,-Sin[b t],Cos[b t]}}


(*(Moebius Geometry *)
(* Many of the following functions make sense in 3-dimensional Moebius geometry only; dim =5,ind= 1. *)

mgcurve[cu_][t_]:=confeuklid[cu[t]];

mfre3D[k_, h_] := {{0, k, 1, 0, 0}, {1, 0, 0, 0, k}, {0, 0, 0, -h, 1}, {0,
    0, h, 0, 0}, {0, 1, 0, 0, 0}};


End[]	

Protect["conemap","confeuklid","confhyp", "euklid","hline", "hplane3pts","hyp3D", 
"hypklein", "hypunit","io", "iob","iobas", "iomatrix", "iohsphereve", "ioradius", 
"iocenter", "mfre3D", "mgcurve", "psobas", "psomatrix", "transio", 
"transoi","sphmap", "euclidpt", "sphhyp3D", "cchyp", "grhyp", "ccell", "grell", "ptell",
 "confell", "trl"] 

EndPackage[]

